/*
  @Copyright 2006-2009 TomTom, BV
  TomTom is hiring experienced mozilla developers! - contact us at xul-jobs@tomtom.com
*/


function BrandingManager()
{
    this.themeProperties = { __proto__ : null };
    this._observers = new Array();
}

BrandingManager.prototype =
{
    _extensionManager : null,
    _observers : null,
    themeProperties : null,

    start : function()
    {
        this._extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
        gApplication.deviceManager.addObserver(this);
        this.changed(true);
    },

    get isUserChosenTheme()
    {
        try
        {
            return prefService().getBoolPref("tomtom.branding.userChoseTheme");
        }
        catch (e)
        {
            return false;
        }
    },
changed : function(deviceChanged, installLocationChanged)
    {
        if (!deviceChanged || !gApplication.deviceManager.activeDevice)
            return;

        var dev = gApplication.deviceManager.activeDevice;
        var brandingFile = dev.brandingFile;

        try
        {
            if (brandingFile)
            {
                let deviceTheme = new ttTheme();
                try {
                    deviceTheme.importFromLocalFile(brandingFile);
                }
                catch (e2)
                {
                    if (!deviceTheme.idstr)
                    {
                        unimportantError(e2);
                        return;
                    }
                }
                let installedTheme = this._getThemeByID(deviceTheme.idstr);
                if (installedTheme && installedTheme.enabled)
                {
                    this._readBrandingConfiguration(installedTheme);
                    this._notifyChange();
                }

                let doInstall = true;
                if (installedTheme && installedTheme.isCompatible)
                {
                    doInstall = (deviceTheme.isCompatible && versionNewerThan(deviceTheme.version, installedTheme.version) && !this.isUserChosenTheme);
                }
                let askServer = (doInstall && !deviceTheme.isCompatible);

                if (askServer)
                {
                    let getNewestVersionsBody = <getnewestversions>
                                                    <contentidlist>
                                                        <content idurl={deviceTheme.idurlWithVersion}/>
                                                    </contentidlist>
                                                </getnewestversions>;
                    let serverargs = {
                        price: "free",
                        includeMissing: false,
                        operatingSystem: gApplication.settings.operatingSystemServerName,
                        deviceModel: dev.model.id,
                        deviceCapabilities: gApplication.deviceManager.activeDeviceCapabilitiesString,
                        deviceParameters: gApplication.deviceManager.activeDeviceParametersString,
                        deviceNavcoreVersion: dev.versionNavCore,
                        deviceNavcoreBuild: dev.buildNumberNavCore
                    };
                    rpc.call("shop", "getNewestVersions", serverargs,
                        getNewestVersionsBody, RPC.METHOD_POST | RPC.USE_CACHE,
                        makeCallback(this, function(result)
                        {
                            if (gApplication.deviceManager.activeDevice != dev)
                                return;

                            let items = gApplication.itemFactory.createItemListFromServer(result);
                            items = items.filter(function(item) item instanceof ttTheme && item.idstr == deviceTheme.idstr);
                            if (items.length > 0)
                            {
                                items[0].download(
                                    function() {},
                                    makeCallback(this, function() {
                                        try
                                        {
                                            prefService().setBoolPref("tomtom.branding.userChoseTheme", false);
                                        }
                                        catch (e)
                                        {
                                            unimportantError(e);
                                        }
                                        this._alertAndRestart();
                                    }),
                                    unimportantError);
                            }
                        }), unimportantError);
                }
                else if (doInstall)
                {
                    deviceTheme.install();
                    try
                    {
                        prefService().setBoolPref("tomtom.branding.userChoseTheme", false);
                    }
                    catch (e)
                    {
                        unimportantError(e);
                    }
                    this._alertAndRestart();
                }
                else if (!this.isUserChosenTheme && !installedTheme.enabled)
                {
                    installedTheme.setEnabled(true);
                    this._alertAndRestart();
                }
            }
            else
            {
                if (ttTheme.restoreCustomTheme())
                    this._alertAndRestart();
            }
        } catch (e) { unimportantError(e); }
    },

    _readBrandingConfiguration : function(theme)
    {
        this.themeProperties = { __proto__ : null };

        var sourceFile = this._extensionManager.getInstallLocation(theme.idstr).getItemFile(theme.idstr, "/chrome/config.xml");

        if (sourceFile && sourceFile.exists())
        {
            var configFile = readFile(sourceFile).join("\n");
            var parsedXML = new XML(configFile.replace(/<\?xml[^>]*\?>/, ""));
            for each (let property in parsedXML.elements())
                this.themeProperties[property.name()] = property.text();
        }
    },
_getThemeByID : function(themeID)
    {
        var extension = this._extensionManager.getItemForID(themeID);

        if (extension && extension.type == CI.nsIUpdateItem.TYPE_THEME)
        {
            var theme = new ttTheme();
            theme.importFromExtensionManager(extension);
            return theme;
        }

        return null;
    },
_alertAndRestart : function()
    {
        var stringBundle = getStringBundle("chrome://tthome/locale/logic/branding.properties");
        var title = stringBundle.GetStringFromName("restart.prompt.title");
        var message = stringBundle.GetStringFromName("restart.prompt.message");
        ttMessageService().alert(title, message);

        runAsync(function()
        {
            Cc["@mozilla.org/toolkit/app-startup;1"].getService(CI.nsIAppStartup)
                .quit(CI.nsIAppStartup.eAttemptQuit | CI.nsIAppStartup.eRestart);
        });
    },

    addObserver : function(observer)
    {
        this._observers.push(observer);
    },

    _notifyChange : function()
    {
        for each (var observer in this._observers)
        {
            try {
                observer.brandingChanged();
            } catch (e) { unimportantError(e); }
        }
    }
}
