/*
  @Copyright 2006-2009 TomTom, BV
  TomTom is hiring experienced mozilla developers! - contact us at xul-jobs@tomtom.com
*/


EXPORTED_SYMBOLS.push(
    "readFile",
    "writeStringToFile",
    "ensureDirExists",
    "moveFile",
    "getFilesInDir",
    "getFile",
    "removeFile",
    "createUniqueDir",
    "createUniqueFile",
    "FileError",
    "FileRemoveError",
    "DirEntryAccessError",
    "GetFileError",
    "FileRemoveError",
    "FileMoveError",
    "CreateUniqueError",
    "CreateDirError",
    "ReadFileError",
    "WriteFileError"
);
function readFile(file)
{
    const MODE_RDONLY = 0x01;
    const PERMS_FILE = 0644;
    assert(file instanceof CI.nsIFile, "file must be nsIFile");
    try {
        var fis = Cc["@mozilla.org/network/file-input-stream;1"]
            .createInstance(CI.nsIFileInputStream);
        fis.init(file, MODE_RDONLY, PERMS_FILE, false);
        try {
            var lis = fis.QueryInterface(CI.nsILineInputStream);

            var result = new Array();
            var line = {value: null};
            var haveMore;
            do
            {
                haveMore = lis.readLine(line)
                result.push(line.value);
            } while (haveMore)
        } finally {
            fis.close();
        }
        return result;
    } catch (e) {
        throw new ReadFileError(e, file);
    }
}
function writeStringToFile(aFile, aString)
{
    assert(aFile instanceof CI.nsIFile, "aFile must be nsIFile");
    try {
        var fileOS = Cc["@mozilla.org/network/file-output-stream;1"]
                     .createInstance(CI.nsIFileOutputStream);
        fileOS.init(aFile, 0x02 | 0x08 | 0x20, 0664, 0);

        try {
            var convOS = Cc["@mozilla.org/intl/converter-output-stream;1"]
                      .createInstance(CI.nsIConverterOutputStream);
            convOS.init(fileOS, "UTF-8", 0, 0x0000);

            try {
                convOS.writeString(aString);
            } finally {
                convOS.close();
            }
        } finally {
            fileOS.close();
        }
    } catch (e) {
        throw new WriteFileError(e, aFile);
    }
}
function ensureDirExists(dir)
{
    assert(dir instanceof CI.nsIFile, "dir must be nsIFile");
    try {
        if (!dir.exists())
            dir.create(CI.nsIFile.DIRECTORY_TYPE, 0775);
        return dir;
    } catch (e) {
        throw new CreateDirError(e, dir);
    }
}
function moveFile(file, dir, name)
{
    assert(file instanceof CI.nsIFile, "file must be nsIFile");
    assert(!dir || dir instanceof CI.nsIFile, "dir must be nsIFile");
    assert(dir || name, "At least one of the parameters dir and name must be specified");
    try
    {
        file.moveTo(dir, name);
    }
    catch (e)
    {
        throw new FileMoveError(e, file, dir, name)
    }
}
function getFilesInDir(dir)
{
    assert(dir instanceof CI.nsIFile, "dir must be nsIFile");
    if (dir.exists() && dir.isDirectory())
    {
        try
        {
            var dirEntries = dir.directoryEntries;
            while (dirEntries.hasMoreElements())
            {
                let entry = dirEntries.getNext().QueryInterface(CI.nsIFile);
                yield entry;
            }
        }
        catch (e)
        {
            throw new DirEntryAccessError(e, dir);
        }
    }
}
function getFile(path)
{
    try
    {
        var file = Cc["@mozilla.org/file/local;1"].createInstance(CI.nsILocalFile);
        file.initWithPath(path);
        return file;
    }
    catch (e)
    {
        throw new GetFileError(e, path);
    }
}
function removeFile(file, recursive)
{
    assert(file instanceof CI.nsIFile, "file must be nsIFile");
    try
    {
        file.remove(recursive);
    }
    catch(e)
    {
        throw new FileRemoveError(e, file, recursive);
    }
}
function createUniqueDir(dir)
{
    assert(dir instanceof CI.nsIFile, "dir must be nsIFile");
    try
    {
        dir = dir.clone();
        dir.createUnique(CI.nsIFile.DIRECTORY_TYPE, 0755);
        return dir;
    }
    catch (e)
    {
        throw new CreateUniqueError(e, dir);
    }
}
function createUniqueFile(file)
{
    assert(file instanceof CI.nsIFile, "file must be nsIFile");
    try
    {
        file = file.clone()
        file.createUnique(CI.nsIFile.NORMAL_FILE_TYPE, 0644);
        return file;
    }
    catch (e)
    {
        throw new CreateUniqueError(e, file);
    }
}
function FileError(error, file)
{
    this.code = error instanceof CI.nsIException ? error.result : error.code;
    this.inner = error;
    this.filePath = file instanceof CI.nsIFile ? file.path : file;
    Exception.call(this, null, (this._name == "FileError" ? this._name : "FileError/" + this._name) + (this.code ? "/" + this.code : ''));
    this.name = "FileError";
}
FileError.prototype =
{
    _name: "FileError",
    operation : "file operation",
    code : null,

    filePath : null,

    get message()
    {
        var stringBundle = getStringBundle("chrome://tthome/locale/logic/fileUtils.properties");
        try
        {
            return stringBundle.formatStringFromName(this._name, [this.filePath, NetUtil.errorMessageForException(this.inner)], 2);
        }
        catch (e)
        {
            return stringBundle.formatStringFromName("FileError", [this.filePath, NetUtil.errorMessageForException(this.inner)], 2);
        }
    },

    get debugInfo()
    {
        return "Performing " + this.operation + " on " + this.filePath;
    }
};
extend(FileError, Exception);
function DirEntryAccessError(error, dir)
{
    FileError.call(this, error, dir);
}
DirEntryAccessError.prototype =
{
    _name : "DirEntryAccessError",
    operation : "iteration over directory entries"
};
extend(DirEntryAccessError, FileError);
function GetFileError(error, path)
{
    FileError.call(this, error, path);
}
GetFileError.prototype =
{
    _name : "GetFileError",
    operation : "file initialization from path"
}
extend(GetFileError, FileError);
function FileRemoveError(error, file, recursive)
{
    this.recursive = recursive;
    FileError.call(this, error, file);
}
FileRemoveError.prototype =
{
    _name : "FileRemoveError",
    operation : "file remove",
    recursive : null,

    get debugInfo()
    {
        return FileError.prototype.__lookupGetter__("debugInfo").apply(this) + (this.recursive ? " recursively" : "");
    }
}
extend(FileRemoveError, FileError);
function FileMoveError(error, file, dir, fileName)
{
    FileError.call(this, error, file);
    this.fileName = fileName;
    if (dir instanceof CI.nsIFile)
        this.dirPath = dir.path;
}
FileMoveError.prototype =
{
    _name: "FileMoveError",
    operation : "file move",

    dirPath : null,
    fileName : null,

    get message()
    {
        var stringBundle = getStringBundle("chrome://tthome/locale/logic/fileUtils.properties");
        try
        {
            if (this.dirPath)
                return stringBundle.formatStringFromName(this._name + ".move", [this.filePath, this.dirPath, NetUtil.errorMessageForException(this.inner)], 3);
            else
                return stringBundle.formatStringFromName(this._name + ".rename", [this.filePath, this.fileName, NetUtil.errorMessageForException(this.inner)], 3);
        }
        catch (e)
        {
            return stringBundle.formatStringFromName("FileError", [this.filePath, NetUtil.errorMessageForException(this.inner)], 2);
        }
    },

    get debugInfo()
    {
        return FileError.prototype.__lookupGetter__("debugInfo").apply(this)
            + "\n(moving to " + this.dirPath + " with name " + this.fileName + ")";
    }
};
extend(FileMoveError, FileError);
function CreateUniqueError(error, dir)
{
    FileError.call(this, error, dir);
}
CreateUniqueError.prototype =
{
    _name : "CreateUniqueError",
    operation : "creating unique file / folder"
}
extend(CreateUniqueError, FileError);
function CreateDirError(error, file)
{
    FileError.call(this, error, file);
}
CreateDirError.prototype =
{
    _name : "CreateDirError",
    operation : "directory creation"
}
extend(CreateDirError, FileError);
function ReadFileError(error, file)
{
    FileError.call(this, error, file);
}
ReadFileError.prototype =
{
    _name : "ReadFileError",
    operation : "reading from file"
};
extend(ReadFileError, FileError);
function WriteFileError(error, file)
{
    FileError.call(this, error, file)
}
WriteFileError.prototype =
{
    _name : "WriteFileError",
    operation : "writing to file"
};
extend(WriteFileError, FileError);
