/*
  @Copyright 2006-2009 TomTom, BV
  TomTom is hiring experienced mozilla developers! - contact us at xul-jobs@tomtom.com
*/


EXPORTED_SYMBOLS.push(
    "ttAddon",
    "XPInstallError"
);
function ttAddon()
{
    ttItem.apply(this);
}
ttAddon.INSTALL_OK = 1;
ttAddon.INSTALL_NO = 2;
ttAddon.INSTALL_NO_OLD_ADDON = ttAddon.INSTALL_NO + 4;
ttAddon.INSTALL_NO_OLD_HOME =  ttAddon.INSTALL_NO + 8;

ttAddon.prototype =
{
    _updateItem: null,
    _sourceFile : null,
    _installManifest : null,
    TOOLKIT_ID : "toolkit@mozilla.org",
    APPLICATION_ID : Cc["@mozilla.org/xre/app-info;1"].getService(CI.nsIXULAppInfo).ID,
canBeInstalled : function(updateItem)
    {
        assert(updateItem, "updateItem cannot be null");
        assert(updateItem instanceof CI.nsIUpdateItem, "updateItem has to be an instance of nsIUpdateItem");

        var minVersion = updateItem.minAppVersion;
        var maxVersion = updateItem.maxAppVersion;
        var currentVersion;

        if (updateItem.targetAppID == this.APPLICATION_ID)
        {
            currentVersion = gApplication.settings.version;
        }
        else if (updateItem.targetAppID == this.TOOLKIT_ID)
        {
            currentVersion = Cc["@mozilla.org/xre/app-info;1"].getService(CI.nsIXULAppInfo).platformVersion;
        }
        else
        {
            return ttAddon.INSTALL_NO;
        }

        var comparator = Cc["@mozilla.org/xpcom/version-comparator;1"].getService(CI.nsIVersionComparator);
        var result = ttAddon.INSTALL_OK;

        if (comparator.compare(minVersion, currentVersion) > 0)
            result = ttAddon.INSTALL_NO_OLD_HOME;
        if (comparator.compare(maxVersion, currentVersion) < 0)
            result = ttAddon.INSTALL_NO_OLD_ADDON;

        return result;
    },
get isCompatible()
    {
        if (this._updateItem)
            return memoize(this, "isCompatible", Boolean(this.canBeInstalled(this._updateItem) & ttAddon.INSTALL_OK)) ;
        else
            return memoize(this, "isCompatible", true);
    },
get sourceFile()
    {
        return this._sourceFile;
    },
get updateItem()
    {
        return this._updateItem;
    },
get installManifest()
    {
        if (!this._installManifest)
        {
            var id = (this._updateItem ? this._updateItem.id : this._idstr);
            assert(id, "Attempted to get install manifest for item without ID");

            var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
            var file = extensionManager.getInstallLocation(id).getItemFile(id, "install.rdf");
            if (!file.exists())
                return null;

            var rdfService = Cc["@mozilla.org/rdf/rdf-service;1"].getService(CI.nsIRDFService);
            var url = getIOService().newFileURI(file);
            this._installManifest = rdfService.GetDataSourceBlocking(url.spec);
        }
        return this._installManifest;
    },
get enabled()
    {
        return (this._getRDFLiteralProperty("isDisabled") != "true");
    },
setEnabled: function(value)
    {
        var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
        if(value)
            extensionManager.enableItem(this._idstr)
        else
            extensionManager.disableItem(this._idstr)
    },
get locked()
    {
        return (this._getRDFLiteralProperty("locked") == "true");
    },
_getRDFLiteralProperty: function(propertyName)
    {
        var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
        var ds = extensionManager.datasource;
        try {
            var rdfService = Cc["@mozilla.org/rdf/rdf-service;1"].getService(CI.nsIRDFService);
            var source = rdfService.GetResource("urn:mozilla:item:" + this._idstr);
            var link = rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#" + propertyName);
            var target = ds.GetTarget(source, link, true);
            if (target)
                return target.QueryInterface(CI.nsIRDFLiteral).Value;
            else
                return null;
        } catch (e) {
            unimportantError(e);
            return null;
        }
    },
importFromExtensionManager: function(item)
    {
        assert(item instanceof CI.nsIUpdateItem, "item must be an nsIUpdateItem");

        this._updateItem = item;

        try
        {
            this.readLocalMetadata();
        } catch (e) { unimportantError(e); }

        this._locationType = 2;
        this._isServer = false;
        this._idstr = makeSafe.contentidstr(item.id);
        this._name = makeSafe.label(item.name);
        var version = item.version.replace(/[^\d.].*/g, "");
        if (version != item.version)
        {
            var warning = Cc["@mozilla.org/scripterror;1"].createInstance(CI.nsIScriptError);
            warning.init("Warning to author: Extension " + this._name + " uses unsupported version scheme, " +
                "converted " + item.version + " into " + version, null, null, 0, 0,
                CI.nsIScriptError.warningFlag   , "HOME message");
            consoleService().logMessage(warning);
        }
        this._version = makeSafe.version(version);

        if (item.iconURL)
            this._icon = new ttImage(getIOService().newURI(item.iconURL, null, null).spec);
        var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
        var ds = extensionManager.datasource;
        try {
            var rdfService = Cc["@mozilla.org/rdf/rdf-service;1"].getService(CI.nsIRDFService);
            var source = rdfService.GetResource("urn:mozilla:item:" + this._idstr);
            var link = rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#description");
            var descriptionTarget = ds.GetTarget(source, link, true);
            if (descriptionTarget)
                this._description = descriptionTarget.QueryInterface(CI.nsIRDFLiteral).Value;
        } catch (e) { unimportantError(e); }

        if (!this._sourceFile)
        {
            try
            {
                var installLocation = extensionManager.getInstallLocation(this._idstr);
                var dir = installLocation.getItemLocation(this._idstr);
                this._size = this._getDirSize(dir);
                this._locations = [{url: dir, size: this._size, content: null}];
            } catch (e) { unimportantError(e); }
        }
        else
        {
            this._size = this._sourceFile.fileSize;
            this._locations = [{url: this._sourceFile.path, size: this._size, content: null}];
        }
    },
download: function(aProgressCallback, aSuccessCallback, aErrorCallback)
    {
        assert(this._locations.length <= 1, "Got more than one download location for an add-on");

        if (!this._locations.length)
        {
            aErrorCallback(this._stringBundle.GetStringFromName("nothingToDownload"));
            return new NoOpAbortable();
        }

        var updateItem = Cc["@mozilla.org/updates/item;1"].createInstance(CI.nsIUpdateItem);
        updateItem.init(this._idstr, "", "app-profile", "", "", this.name, this._locations[0].url.spec, "", this.icon ? this.icon.url : "", "", "", this._listEntry.extensionManagerType, "");

        aProgressCallback(0, this._stringBundle.GetStringFromName("item.startDownload"));

        var me = this;
        var downloader = new AddonDownloader();
        downloader.download(updateItem, this._locations[0].size, aProgressCallback, function(updateItem)
        {
            try {
                me.updateLocalMetadata();
            } catch (e) { unimportantError(e); }

            var result = new me._listEntry.modelType();
            result.importFromExtensionManager(updateItem);
            aSuccessCallback(result);
        }, aErrorCallback);

        return downloader;
    },
uninstall: function(aProgressCallback, aSuccessCallback, aErrorCallback)
    {
        try
        {
            var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
            extensionManager.uninstallItem(this._idstr);
            aSuccessCallback();
        }
        catch (e)
        {
            aErrorCallback(e);
        }
        return new NoOpAbortable();
    },
canWriteToLocation: function()
    {
        if (!this.isLocal)
            return false;

        var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
        var installLocation = extensionManager.getInstallLocation(this._idstr);
        return (installLocation ? installLocation.canAccess : false);
    },
updateLocalMetadata: function()
    {
        if (!this._xml)
            return;

        var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
        var rdfService = Cc["@mozilla.org/rdf/rdf-service;1"].getService(CI.nsIRDFService);

        var ds = extensionManager.datasource;
        var source = rdfService.GetResource("urn:mozilla:item:" + this._idstr);
        var link = rdfService.GetResource("http://www.tomtom.com/2007/em-rdf#homeXML");

        var oldTarget = ds.GetTarget(source, link, true);
        if (oldTarget)
            ds.Unassert(source, link, oldTarget);

        var target = rdfService.GetLiteral(this._xml.toXMLString());
        ds.Assert(source, link, target, true);
        ds.QueryInterface(CI.nsIRDFRemoteDataSource).Flush();
    },
readLocalMetadata: function()
    {
        if (!this._updateItem)
            return;

        var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
        var rdfService = Cc["@mozilla.org/rdf/rdf-service;1"].getService(CI.nsIRDFService);

        var ds = extensionManager.datasource;
        var source = rdfService.GetResource("urn:mozilla:item:" + this._updateItem.id);
        var link = rdfService.GetResource("http://www.tomtom.com/2007/em-rdf#homeXML");
        var target = ds.GetTarget(source, link, true);

        var installManifest = this.installManifest;
        var installManifestRoot = rdfService.GetResource("urn:mozilla:install-manifest");
        if (target)
        {
            if (installManifest && !ttAddon.getOperationPending(this._updateItem.id))
            {
                var oldTarget = installManifest.GetTarget(installManifestRoot, link, true);
                if (oldTarget)
                    installManifest.Unassert(installManifestRoot, link, oldTarget);

                installManifest.Assert(installManifestRoot, link, target, true);
                ds.Unassert(source, link, target);

                installManifest.QueryInterface(CI.nsIRDFRemoteDataSource).Flush();
                ds.QueryInterface(CI.nsIRDFRemoteDataSource).Flush();
            }
        }
        else
        {
            target = installManifest.GetTarget(installManifestRoot, link, true);
        }

        if (target)
        {
            var xml = new XML(target.QueryInterface(CI.nsIRDFLiteral).Value);
            this.importFromXML(xml);
        }

        var previewLink = rdfService.GetResource("http://www.tomtom.com/2007/em-rdf#previewURL");
        var previewTarget = installManifest.GetTarget(installManifestRoot, previewLink, true);
        if (previewTarget)
        {
            try {
                var previewURL = previewTarget.QueryInterface(CI.nsIRDFLiteral).Value;
                this._preview = new ttImage(getIOService().newURI(previewURL, null, null).spec);
            } catch (e) { unimportantError(e); }
        }
    },
importFromLocalFile : function(file)
    {
        assert(file instanceof CI.nsIFile, "need a file to import from");

        assert(this.listEntry && "extensionManagerType" in this.listEntry && "modelType" in this.listEntry, "Can only install subtype of addon");
        let installManifestData = "";
        try
        {
            let zipReader = Cc["@mozilla.org/libjar/zip-reader;1"].createInstance(CI.nsIZipReader);
            zipReader.open(file);

            let inputStream = Cc["@mozilla.org/scriptableinputstream;1"].createInstance(CI.nsIScriptableInputStream);
            inputStream.init(zipReader.getInputStream("install.rdf"));

            while (true)
            {
                let text = inputStream.read(1024);
                if (!text)
                    break;
                installManifestData += text;
            }
            inputStream.close();
        }
        catch (e)
        {
            throw new ReadFileError(e, file);
        }

        this._sourceFile = file;
        var sourceFileURI = getIOService().newFileURI(this._sourceFile);
        var rdfService = Cc["@mozilla.org/rdf/rdf-service;1"].getService(CI.nsIRDFService);

        this._installManifest = Cc["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"].createInstance(CI.nsIRDFDataSource);
        Cc["@mozilla.org/rdf/xml-parser;1"].createInstance(CI.nsIRDFXMLParser)
                    .parseString(this._installManifest, sourceFileURI, installManifestData);

        var installManifestRoot = rdfService.GetResource("urn:mozilla:install-manifest");
        var me = this;
        var getLiteralInResource = function(target, property)
        {
            let resource = rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#" + property);
            return me.installManifest.GetTarget(target, resource, true)
                .QueryInterface(CI.nsIRDFLiteral).Value;
        };
        var getLiteral = function(property)
        {
            return getLiteralInResource(installManifestRoot, property);
        };

        var resource = rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#targetPlatform");
        var targetPlatforms = null;

        try {
            targetPlatforms = this._installManifest.GetTargets(installManifestRoot, resource, true);
        } catch (e) {
        }

        if (targetPlatforms != null && targetPlatforms.hasMoreElements())
        {
            let foundMatchingOS = false;
            let foundMatchingOSAndABI = false;
            let requireABICompatibility = false;

            while (targetPlatforms.hasMoreElements())
            {
                let targetPlatform = targetPlatforms.getNext().QueryInterface(CI.nsIRDFLiteral).Value;

                let os = targetPlatform.split("_")[0];
                let index = targetPlatform.indexOf("_");
                let abi = index != -1 ? targetPlatform.substr(index + 1) : null;

                if (os == gApplication.settings.operatingSystemTag)
                {
                    foundMatchingOS = true;
                    if (abi != null)
                    {
                        requireABICompatibility = true;
                        if (abi == gApplication.settings.XPCOMABI && abi != "unknownABI")
                        {
                            foundMatchingOSAndABI = true;
                            break;
                        }
                    }
                }
            }

            if (!foundMatchingOS || (requireABICompatibility && !foundMatchingOSAndABI))
            {
                let stringBundle = getStringBundle("chrome://tthome/locale/logic/model/ttAddon.properties");
                throw new Exception(stringBundle.GetStringFromName("addonIncompatibilityPlatformMessage"));
            }

        }
        resource = rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#targetApplication");
        var targetApps = me.installManifest.GetTargets(installManifestRoot, resource, true);
        let [targetApp, minVersion, maxVersion] = ["", "", ""];
        while (targetApps.hasMoreElements())
        {
            let appInfo = targetApps.getNext();
            if (appInfo instanceof CI.nsIRDFResource)
            {
                try
                {
                    var id = getLiteralInResource(appInfo, "id");
                    if (id == this.APPLICATION_ID || (id == this.TOOLKIT_ID && targetApp == ""))
                    {
                        targetApp = id;
                        minVersion = getLiteralInResource(appInfo, "minVersion");
                        maxVersion = getLiteralInResource(appInfo, "maxVersion");
                    }
                } catch (e) { unimportantError(e); }
            }
        }

        var updateItem = Cc["@mozilla.org/updates/item;1"].createInstance(CI.nsIUpdateItem);
        updateItem.init(getLiteral("id"), getLiteral("version"), "app-profile", minVersion, maxVersion, getLiteral("name"), "", "", "", "", "", CI.nsIUpdateItem.TYPE_THEME, targetApp);

        let result = this.canBeInstalled(updateItem);

        this.importFromExtensionManager(updateItem);

        this._locations = [{url : sourceFileURI, size : this._size, content : "main"}];

        if (Boolean(result & ttAddon.INSTALL_NO))
        {
            let stringBundle = getStringBundle("chrome://tthome/locale/logic/model/ttAddon.properties");

            if (result == ttAddon.INSTALL_NO_OLD_HOME)
                throw new Exception(stringBundle.GetStringFromName("addonIncompatibilityMessageOldHOME"));
            if (result == ttAddon.INSTALL_NO_OLD_ADDON)
                throw new Exception(stringBundle.GetStringFromName("addonIncompatibilityMessageOldAddon"));

            throw new Exception(stringBundle.GetStringFromName("addonIncompatibilityMessage"));
        }
    },
install : function()
    {
        assert(this._locationType == 2 && this._sourceFile != null, "Can only install add-ons that are associated with a file.");{}
var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
        extensionManager.installItemFromFile(this._sourceFile, "app-profile");
    },
_getDirSize: function(dir)
    {
        var size = 0;
        if (dir.exists())
        {
            if (dir.isDirectory())
            {
                for (let entry in getFilesInDir(dir))
                    size += this._getDirSize(entry);
            }
            else
                size = dir.fileSize;
        }
        return size;
    },

  __end : null
}
extend(ttAddon, ttItem);
ttAddon.prototype.importItemsFromExtensionManager = function()
{
    assert(this.listEntry && "extensionManagerType" in this.listEntry, "importItemsFromExtensionManager called for a class that doesn't have extension manager type defined");

    var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
    var updateItems = extensionManager.getItemList(this.listEntry.extensionManagerType, {});
    var result = [];
    for (var i = 0; i < updateItems.length; i++)
    {
        try {
            var item = new this.listEntry.modelType();
            item.importFromExtensionManager(updateItems[i]);
            result.push(item);
        } catch (e) { unimportantError(e); }
    }
    return result;
}
ttAddon.prototype.importItemFromExtensionManager = function(id)
{
    assert(this.listEntry && "extensionManagerType" in this.listEntry, "importItemsFromExtensionManager called for a class that doesn't have extension manager type defined");

    var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager),
        extension = extensionManager.getItemForID(id);

    var result = null;

    if (extension && extension.type == this._listEntry.extensionManagerType)
    {
        try {
            result = new this._listEntry.modelType();
            result.importFromExtensionManager(extension);
        } catch (e) { unimportantError(e); }
    }

    return result;
}
ttAddon.isRestartRequired = function()
{
    var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
    var items = extensionManager.getItemList(CI.nsIUpdateItem.TYPE_ANY, {});
    for (var i = 0; i < items.length; i++)
    {
        try {
            if (ttAddon.getOperationPending(items[i].id))
                return true;
        } catch (e) { unimportantError(e); }
    }
    return false;
}

ttAddon._operationsMap = {
    "needs-install": "install",
    "needs-upgrade": "upgrade",
    "needs-uninstall": "uninstall",
    "needs-enable": "enable",
    "needs-disable": "disable"
};
ttAddon.getOperationPending = function(id)
{
    var ds = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager).datasource;
    var rdfService = Cc["@mozilla.org/rdf/rdf-service;1"].getService(CI.nsIRDFService);
    var source = rdfService.GetResource("urn:mozilla:item:" + id);

    var opLink = rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#opType");
    var opTarget = ds.GetTarget(source, opLink, false);

    if (opTarget && opTarget.QueryInterface(CI.nsIRDFLiteral).Value in ttAddon._operationsMap)
        return ttAddon._operationsMap[opTarget.QueryInterface(CI.nsIRDFLiteral).Value];

    var nameLink = rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#internalName");
    var nameTarget = ds.GetTarget(source, nameLink, true);
    if (nameTarget && nameTarget.QueryInterface(CI.nsIRDFLiteral).Value)
    {
        var themeName = nameTarget.QueryInterface(CI.nsIRDFLiteral).Value;
        var lastSelectedTheme = null;
        var switchingTheme = false;
        try {
            switchingTheme = prefService().getBoolPref("extensions.dss.switchPending");
            lastSelectedTheme = prefService().getCharPref("extensions.lastSelectedSkin");
        } catch(e) {}
        if (switchingTheme && themeName == lastSelectedTheme)
            return ttAddon._operationsMap["needs-enable"];
    }

    return null;
}
ttAddon.restartCommitFunction = function(forceRestart, progressCallback, successCallback, errorCallback)
{
    var stringBundle = getStringBundle("chrome://tthome/locale/logic/model/ttAddon.properties");
    if (ttAddon.isRestartRequired())
    {
        var restart = true;
        if (!forceRestart)
        {
            var brandBundle = getStringBundle("chrome://branding/locale/brand.properties");
            var shortName = brandBundle.GetStringFromName("brandFullName");
            var title = stringBundle.GetStringFromName("restart.prompt.title");
            var message = stringBundle.formatStringFromName("restart.prompt.message", [shortName], 1);
            restart = promptService().confirm(getMainWindowByType(), title, message);
        }
        if (restart)
        {
            Cc["@mozilla.org/toolkit/app-startup;1"].getService(CI.nsIAppStartup)
                    .quit(CI.nsIAppStartup.eAttemptQuit | CI.nsIAppStartup.eRestart);
        }
    }
    successCallback(stringBundle.GetStringFromName("commit.restart.done"));

    return new NoOpAbortable();
}
function AddonDownloader()
{
}
AddonDownloader.prototype =
{
    _stringBundle : getStringBundle("chrome://tthome/locale/logic/model/ttAddon.properties"),
    _updateItem: null,
    _manager: null,
    _progressCallback: null,
    _successCallback: null,
    _errorCallback: null,
    _finishedCallback: null,
    _error: null,
    _defaultSize: 0,
    _startTime: 0,
download: function(updateItem, defaultSize, progressCallback, successCallback, errorCallback)
    {
        assert(!this._manager, "download already started");

        this._updateItem = updateItem;
        this._progressCallback = progressCallback;
        this._successCallback = successCallback;
        this._errorCallback = errorCallback;

        this._manager = Cc["@mozilla.org/xpinstall/install-manager;1"].createInstance(CI.nsIXPInstallManager);
        this._manager.initManagerFromChrome([updateItem.xpiURL], 1, this);

        this._error = null;
        this._defaultSize = defaultSize;
        this._startTime = Date.now();

        var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
        extensionManager.addDownloads([updateItem], 1, this._manager);
    },
cancel: function(ex)
    {
        assert(this._manager, "cancelling a download that hasn't been started yet?");

        if (!ex)
            ex = new UserCancelledException();
        this._error = ex;
        this._manager.QueryInterface(CI.nsIObserver).observe(null, "xpinstall-progress", "cancel");

        var stringBundle = getStringBundle("chrome://tthome/locale/logic/model/ttAddon.properties");
        this._progressCallback(-1, stringBundle.GetStringFromName("cancelling"));
    },
setFinishedCallback: function(callback)
    {
        assert(typeof callback == "function", "Function parameter expected");
        this._finishedCallback = callback;
    },
onProgress: function(index, value, maxValue)
    {
        if (maxValue < 0)
            maxValue = this._defaultSize;
        var percentage = (value / maxValue) * 100;
        var remainingTimeText = ProgressTracker.calculateRemainingTime(this._startTime, percentage);
        this._progressCallback(percentage, remainingTimeText, maxValue);
    },
onStateChange: function(index, state, value)
    {
        switch (state)
        {
            case CI.nsIXPIProgressDialog.DOWNLOAD_START:
            case CI.nsIXPIProgressDialog.DOWNLOAD_DONE:
            case CI.nsIXPIProgressDialog.INSTALL_START:
                break;
            case CI.nsIXPIProgressDialog.INSTALL_DONE:
                try
                {
                    var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
                    if (value == 0 || value == 999)
                    {
                        var updateItem = extensionManager.getItemForID(this._updateItem.id);
                        if (!updateItem)
                        {
                            this._errorCallback(this._stringBundle.GetStringFromName("addon.installFailed"));
                            return;
                        }

                        try
                        {
                            this._successCallback(updateItem);
                        }
                        catch (e)
                        {
                            this._errorCallback(e);
                        }
                    }
                    else
                    {
                        var error;
                        if (value == -210 && this._error)
                        {
                            error = this._error;
                        }
                        else
                        {
                            error = new XPInstallError(value, this._updateItem);
                        }
                        this._errorCallback(error);
                    }
                }
                finally
                {
                    this._manager = null;
                    if (this._finishedCallback)
                        this._finishedCallback(this);
                }
                break;
        }
    },

    QueryInterface: function(iid)
    {
        if (iid.equals(CI.nsISupports) ||
            iid.equals(CI.nsIAddonUpdateListener))
        {
            return this;
        }

        throw Components.results.NS_NOINTERFACE;
    }
};
extend(AddonDownloader, NetworkCall);

function XPInstallError(errorCode, updateItem)
{
    this.code = errorCode;
    this.url = updateItem.xpiURL;
    this.addonName = updateItem.name;
    this.addonId = updateItem.id;

    Exception.call(this, null, this._id + "/" + this.code + "/#" + this.url);
    this.name = "XPInstall error";
}
XPInstallError.prototype =
{
    _id: "XPInstall",
    code : null,
    url : null,
    addonName : null,
    addonId : null,
    message: null,

    get message()
    {
        var stringBundle = getStringBundle("chrome://tthome/locale/logic/xpinstall.properties");
        try
        {
            return stringBundle.formatStringFromName("error" + this.code, [this.addonName], 1);
        }
        catch (e)
        {
            return stringBundle.formatStringFromName("unknown.error", [this.code, this.addonName], 2);
        }
    },

    get debugInfo()
    {
        return "Installing add-on: " + this.addonId + " (" + this.addonName + ")\n" +
               "URL: " + this.url;
    }
};
extend(XPInstallError, Exception);
