/*
  @Copyright 2006-2009 TomTom, BV
  TomTom is hiring experienced mozilla developers! - contact us at xul-jobs@tomtom.com
*/


EXPORTED_SYMBOLS.push("ttNavItem");
function ttNavItem()
{
    ttItem.apply(this);

}
ttNavItem.prototype =
{
    _dll : null,
    _drm : null,
    _drmDeviceCode : null,
    _drmScheme : null,
    _drmNeedsKey : false,
    _drmActivationCode : null,
    _targetModels : null,
    _capabilities : null,
    _creatingDeviceID : null,
get sourceDLL()
    {
        return this._dll.QueryInterface(CI.ttINavItem);
    },

    get significantIDs()
    {
        if (gApplication.homeServices.isSingular(this._listEntry.itemtypeEnum))
            return [this.idurl, this._listEntry.xmlTag + "/singular"];

        return ttItem.prototype.__lookupGetter__("significantIDs").apply(this);
    },
get drm()
    {
        return this._drm;
    },
get drmDeviceCode()
    {
        return this._drmDeviceCode;
    },
get drmScheme()
    {
        return this._drmScheme;
    },
get drmActivationCode()
    {
        return this._drmActivationCode;
    },
get drmNeedsKey()
    {
        return this._drmNeedsKey;
    },
    get creatingDeviceID()
    {
        return this._creatingDeviceID;
    },
    get isShareable()
    {
        return ttItem.prototype.__lookupGetter__("isShareable").call(this) &&
            (this.isServer || !this.sourceDLL.isEncrypted);
    },
get capabilities()
    {
         return this._capabilities;
    },
canInstallOn : function(device)
    {
        assert(device, "device must not be null");
        if (this._drm && !this._drmNeedsKey)
        {
            if (device.deviceCode != this._drmDeviceCode)
                return 2;
        }
        if (this._capabilities)
        {
            for each (var capability in this._capabilities)
            {
                if (!device.hasCapability(capability))
                    return 3;
            }
        }
        else if (this._targetModels)
        {
            if ( ! (device.model.id in this._targetModels))
                return 3;
        }
        if (this.locationType != 1 && this.size)
        {
            var safeSize = this.size + 15000000;
            if (safeSize > gApplication.deviceManager.activeInstallLocation.sizeInBytesTotal &&
                !gApplication.deviceManager.activeDevice.hasCapability("SDSlot"))
            {
                return 5;
            }
        }
        if (this.locationType == 2 || this.locationType == 3)
        {
            var reason = this._dll.isInstallableOnDevice(device.id);
            if (reason == CI.ttIItem.kInstallable)
                return 0;
            else if (reason == CI.ttIItem.kDRMDeviceMismatch)
                return 2;
            else if (reason == CI.ttIItem.kDeviceTypeNotSupported)
                return 3;
            else if (reason == CI.ttIItem.kNavCoreVersionTooLow)
                return 4;
            else if (reason == CI.ttIItem.kIncompatibleTTSEngine)
                return 6;
            else
                return 1;
        }
        return 0;
    },
canUseOnThisDevice : function()
    {
        assert(this.locationType == 1, "item must be located on device");

        var reason = this._dll.isUsableOnDevice();
        assert(reason != CI.ttIItem.kNotADeviceItem, "locationType is 'on device' but DLL claims the item is not on device");

        if (reason == CI.ttIItem.kUsable)
            return 0;
        else if (reason == CI.ttIItem.kNotUsableDRMCertificateMissing)
            return 2;
        else if (reason == CI.ttIItem.kIncompatibleTTSEngine)
            return 3;
        else
            return 1;
    },
set creatingDeviceID(newVal)
    {
        this._ensureXML();
        this._xml.supplier.@sourcedevice = newVal;
        this._xml.supplier.@id = "device-" + newVal;
    },
set drmDeviceCode(newVal)
    {
        this._drmDeviceCode = makeSafe.alphanum(newVal)
        this._ensureXML();
        this._xml.contentprotection.@devicecode = this._drmDeviceCode;
    },
set drmNeedsKey(newVal)
    {
        this._drmNeedsKey = makeSafe.bool(newVal)
        this._ensureXML();
        if (this._drmScheme == "activation-code")
            this._xml.contentprotection.activation.@needskey = this._drmNeedsKey;
        else if (this._drmScheme == "durham")
            this._xml.contentprotection.durhamkey.@needskey = this._drmNeedsKey;
    },
set drmActivationCode(newVal)
    {
        assert(this._drmScheme == "activation-code", "DRM scheme must be 'activation-code'");

        this._drmActivationCode = makeSafe.alphanumspace(makeSafe.nonemptystring(newVal));
        this._ensureXML();
        this._xml.contentprotection.activation.@code = this._drmActivationCode;
    },

    importFromXML : function(xml)
    {
        ttItem.prototype.importFromXML.apply(this, arguments);

        if ("contentprotection" in xml)
        {
            this._drm = true;
            this._drmDeviceCode = makeSafe.alphanum(xml.contentprotection.@devicecode);
            this._drmScheme = makeSafe.alphanumdash(xml.contentprotection.@scheme);
            if (this._drmScheme == "durham" && "@needskey" in xml.contentprotection.durhamkey)
                this._drmNeedsKey = makeSafe.bool(xml.contentprotection.durhamkey.@needskey);
            else if (this._drmScheme == "activation-code" && "@needskey" in xml.contentprotection.activation)
                this._drmNeedsKey = makeSafe.bool(xml.contentprotection.activation.@needskey);
            if (this._drmScheme == "activation-code" && !this._drmNeedsKey)
            {
                this._drmActivationCode =
                    makeSafe.alphanumspace(makeSafe.nonemptystring(xml.contentprotection.activation.@code));
            }
        }

        if ("targetdevice" in xml)
        {
            if ("@models" in xml.targetdevice &&
                xml.targetdevice.@models != "all")
            {
                var array = makeSafe.list(xml.targetdevice.@models,
                                function(item) { return makeSafe.alphanumdash(item); },
                                " ");
                this._targetModels = new Object();
                this._targetModels.__proto__ = null;
                for (var i = 0; i < array.length; i++)
                {
                    this._targetModels[array[i]] = true;
                }
            }
            if ("@capabilities" in xml.targetdevice)
            {
                this._capabilities = makeSafe.list(xml.targetdevice.@capabilities,
                                        function(item) { return makeSafe.alphanumdash(item); },
                                        " ");
            }
        }

        if ("supplier" in xml && "@sourcedevice" in xml.supplier)
            this._creatingDeviceID = makeSafe.string(xml.supplier.@sourcedevice);

    },

    importFromServer : function(xml)
    {
        ttItem.prototype.importFromServer.apply(this, arguments);

        if ( !("contentprotection" in xml))
        {
            this._drm = false;
        }
    },
_correctOldXML : function(xml, version)
    {
        if (versionNewerThan("2.2", version))
        {
            for each (var downloadLoc in xml.download.url)
            {
                if ("@size" in downloadLoc && downloadLoc.@size == "")
                    delete downloadLoc.@size;
            }
        }
    },
_createDLLItem : function(createNewInstance)
    {
        var item = this;

        if (this._dll && !createNewInstance)
            return;
        var tocfile = gApplication.settings.osTempDir;
        tocfile.append(Math.ceil(Math.random() * 1000000) + ".toc");
        tocfile = createUniqueFile(tocfile);

        var toc = this.createTOC(getIOService().newFileURI(tocfile));
        writeStringToFile(tocfile, toc.toXMLString());
        var dllitem = gApplication.homeServices.localNavItems().createItemFromTOCFile(tocfile);

        if (createNewInstance)
            item = gApplication.itemFactory.createItemFromDLL(dllitem);
        else
            this._dll = dllitem;

        removeFile(tocfile, false);

        return item;
    },
_ensureSourceDLL : function()
    {
        this._createDLLItem(false);
        assert(this._dll, "must have a DLL representation of item \"" + this.name + "\"", "TTNavItem/EnsureSource/FromTOCCreationFailure");
    },

    importFromDLL : function(item)
    {
        assert(item instanceof CI.ttIItem, "need ttIItem");
        this._dll = item;

        this._idstr = makeSafe.contentidstr(item.id);
        this._name = makeSafe.label(item.name);
        this._description = item.description;
        try {
            this._version = makeSafe.version(item.version);
        } catch (e) {
            unimportantError(e);
            this._version = 0;
        }
        this._size = item.sizeInBytes;
        this._endPrice = null;

        var t = item.locationType;
        if (t == CI.ttIItem.kLocationDevice)
            this._locationType = 1;
        else if (t == CI.ttIItem.kLocationHarddisk)
            this._locationType = 2;
        else if (t == CI.ttIItem.kLocationMedia)
            this._locationType = 3;
        else
            throw new InvalidValue(this._stringBundle.GetStringFromName("unexpectedItemLocation"), t);
        var xml = null;
        var toc = null;

        if (this.locationType == 2 || this.locationType == 3)
        {
            toc = this.getTOCXML();
            var homeData = toc.elements()[0].HOME2;
            if (homeData.length())
            {
                xml = homeData.elements()[0];
                if ("@version" in homeData)
                {
                    this._correctOldXML(xml, makeSafe.version(homeData.@version));
                }
            }
        }
        else if (this.locationType == 1)
        {
            xml = item.QueryInterface(CI.ttINavItem).metadata;
            if (xml)
            {
                xml = new XML(xml);
                let ucVersion = xml.@version;
                xml = xml.elements();
                if (ucVersion.toString().length)
                {
                    this._correctOldXML(xml, makeSafe.version(ucVersion));
                }
            }
        }
        if (xml && xml.length())
            this.importFromXML(xml);

        var location = item.location;
        if (toc && "Location" in toc.*)
        {
            this._locations = [];

            let tocFile = this.sourceDLL.QueryInterface(CI.ttINavItem).tocFile;
            let path = getIOService().newFileURI(tocFile);

            for each (let loc in toc.*.Location)
            {
                this._locations.push({
                    url : getIOService().newURI(loc.text().toString(), null, path),
                    size : 0,
                    content : loc.@content ? makeSafe.string(loc.@content) : null
                });
            }
        }
        else if (location)
        {
            this._locations = [ { url: location, size: item.sizeInBytes, content: null } ];
        }
        else
        {
            this._locations = [];
        }

        if (item.icon)
            this._icon = ttImage.prototype.createFromDLL(item.icon);
        if (item.preview)
            this._preview = ttImage.prototype.createFromDLL(item.preview);

        var dependency = item.dependsOn;
        if (dependency)
        {
            this._dependencies = [ttItemRef.prototype.createFromDLL(dependency)];
        }
        else
        {
            this._dependencies = [];
        }
        if (this.locationType == 1)
            this._size = item.sizeInBytes;
        if (this.locationType == 1 && item.isUsableOnDevice() == CI.ttIItem.kNotUsableDRMCertificateMissing)
        {
            this._drm = true;
            this._drmScheme = "durham";
            this._drmNeedsKey = true;
        }

        this._isServer = false;
    },
createTOC : function(aTOCURL)
    {
        assert(aTOCURL ? aTOCURL instanceof CI.nsIURL : true, "aTOCURL must be a subtype of nsIURL");

        var table = <TomTomToc version="1.0"/>;
        var toc = <{this.listEntry.tocTag}/>;
toc.Id = this.idstr;
        toc.Version = this.version;
        toc.Name = this.name;
        if (this.size)
            toc.Size = this.size;
        if (this.description)
            toc.Description = this.description;

        if (this.listEntry.tocSubtype)
            toc.@subtype = this.listEntry.tocSubtype;

        if (this.idurl)
            toc.Id.@idurl = this.idurl;
        if (this._hasHOME2Metadata)
        {
            var [majorVersion, minorVersion] = gApplication.settings.version.split(".");
            toc.HOME2.@version = majorVersion + "." + minorVersion;
            toc.HOME2.appendChild(this._xml);
        }

        for (let i = 0; i < this._locations.length; i++)
        {
            var locationURL = this._locations[i].url;

            if (locationURL instanceof CI.nsIURI)
            {
                toc.Location[i] = locationURL.spec;
            }
            else if (locationURL instanceof CI.nsIFile)
            {
                var fileURI = getIOService().newFileURI(locationURL);
                toc.Location[i] = this._getTOCRelativeURI(fileURI.spec, aTOCURL);
            }
            else
            {
                throw new Bug("Location URL has to be an instance of nsIURI or nsIFile","CreateTOC/InvalidLocationURLType");
            }

            if (this._locations[i].content)
                toc.Location[i].@content = this._locations[i].content;
        }

        if (this.icon)
            toc.Picture = this._getTOCRelativeURI(this.icon.url, aTOCURL);

        if (this.preview)
            toc.Preview = this._getTOCRelativeURI(this.preview.url, aTOCURL);

        if (this._targetModels && !("all" in this._targetModels))
        {
            var list = "";
            for (var model in this._targetModels)
                list += (list == "" ? "" : ",") + model;
            toc.Target = list;
        }

        if (this._capabilities)
            toc.Capabilities = this._capabilities.join(",");

        if (this._drm)
        {
            if (this._drmDeviceCode)
                toc.InstallRestrictions.@deviceId = this._drmDeviceCode;

            if (this._drmScheme == "activation-code" &&
                this._drmActivationCode)
            {
                toc.ActivationCode = this._drmActivationCode;
            }
        }

        if (this._dependencies && this._dependencies.length)
        {
            assert(this._dependencies.length == 1, "Items with more than one dependency currently not supported");

            var dependency = this._dependencies[0];
            toc.appendChild(<Dependency type={dependency.type.tocTag} id={dependency.idstr}/>);
            if (dependency.idurl)
                toc.Dependency.@idurl = dependency.idurl;
            if (dependency.version)
                toc.Dependency.@version = dependency.version;
        }

        table.appendChild(toc);
        return table;
    },
_getTOCRelativeURI: function (aURI, aTOCURL)
    {
        assert(typeof(aURI) == "string");
        assert(aTOCURL ? aTOCURL instanceof CI.nsIURL : true, "aTOCURL must be a subtype of nsIURL");

        if (!aTOCURL)
            return aURI;

        try
        {
            var url = getIOService().newURI(aURI, null, null);
            if (url instanceof CI.nsIFileURL)
            {
                var relURI = aTOCURL.getRelativeSpec(url);
                if (relURI == "")
                    throw new InvalidValue("URI to make relative must not be equal to TOC file URI.", aURI);
                return relURI;
            }
            else
            {
                return url.spec;
            }
        }
        catch (e)
        {
unimportantError(e);
            return aURI;
        }
    },

    getTOCXML : function ()
    {
        var tocURI = getIOService().newFileURI(this.sourceDLL.QueryInterface(CI.ttINavItem).tocFile);
        try {
            var tocString = readURLUTF8(tocURI).join("\n");
        } catch (e) {
            throw new ExceptionWithCustomMessage(this._stringBundle.formatStringFromName("tocFileNotFound", [ this.name ], 1), e);
        }
        tocString = tocString.replace(/<\?xml[^>]*\?>/, "");
        return new XML(tocString);
    },

    _saveMetadata : function()
    {
        assert(!this.isServer, "Can't save to server item");
        assert(this._dll instanceof CI.ttINavItem, "Must be DLL item of type ttINavItem");
if (!this._hasHOME2Metadata)
        {
return;
        }

        var dllitem = this._dll.QueryInterface(CI.ttINavItem);

        this._ensureXML();
gApplication.itemFactory.createItemFromServer(this._xml.copy());
        var [majorVersion, minorVersion] = gApplication.settings.version.split(".");
        var xmlHOME2 = <HOME2 version={majorVersion + "." + minorVersion}/>;
        xmlHOME2.appendChild(this._xml.copy());
        dllitem.metadata = xmlHOME2.toXMLString();
    },

    saveMetadata : function()
    {
        this._saveMetadata();
        if (this instanceof ttNavItem && this.locationType == 2)
        {
            var dllitem = this._dll.QueryInterface(CI.ttINavItem);
            var tocURI = getIOService().newFileURI(dllitem.tocFile);
            var toc = this.createTOC(tocURI);
            writeStringToFile(dllitem.tocFile, toc.toXMLString());
        }
    },
download : function(aProgressCallback, aSuccessCallback, aErrorCallback)
    {
        if (this._locations == null ||
            this._locations.length == 0)
            throw new Exception(this._stringBundle.GetStringFromName("nothingToDownload"));
        var downloadLocations =
            this._locations.slice(0);

        if (this.preview)
        {
            downloadLocations.unshift({
                url: getIOService().newURI(this.preview.url, null, null),
                size: 10000,
                content: "preview"
            });
        }

        if (this.icon)
        {
            downloadLocations.unshift({
                url: getIOService().newURI(this.icon.url, null, null),
                size: 1500,
                content: "icon"
            });
        }
        var downloadTempFolder =
            gApplication.settings.downloadTempItemsDir;
        var downloadCompleteFolder =
            gApplication.settings.downloadCompleteItemsDir;
        downloadCompleteFolder.append(this.typeID);
        downloadCompleteFolder.append(makeSafeEscape.filename(this.name));
        downloadCompleteFolder = createUniqueDir(downloadCompleteFolder);
        aProgressCallback(0, this._stringBundle.GetStringFromName("item.startDownload"));
        for each (let location in downloadLocations)
            location.actualSize = location.size;

        var result = new ttNavItem();
        var downloader = new Downloader();
        var combinedCall = new CombinedCall();
        var downloadedSoFar = 0;
        var currentIndex = 0;
        var startTime = Date.now();
        var me = this;

        function progressCallback(aProgressPercentage, aProgressTextUnused, aActualSize)
        {

            if (aActualSize)
                downloadLocations[currentIndex].actualSize = aActualSize;
            var totalSize = 0;
            for each (let location in downloadLocations)
                totalSize += location.actualSize;
            var totalPercentage =
                (downloadedSoFar * 100 +
                 aActualSize * aProgressPercentage) /
                totalSize;
            var remainingTimeText =
                ProgressTracker.calculateRemainingTime(
                    startTime,
                    totalPercentage);
            aProgressCallback(
                totalPercentage,
                remainingTimeText,
                totalSize);
        }

        function errorCallback(err)
        {
            var location = downloadLocations[currentIndex];
            var content = location.content;
            if ((content == "icon") ||
                (content == "preview"))
            {
                location.url = null;
                location.actualSize = 0;
                me[content] = null;

                unimportantError(err);
                successCallback(null);
            }
            else
            {
                unimportantError(err);
                downloadCompleteFolder.remove(true);
                aErrorCallback(err);
            }
        }

        function successCallback(aTargetFile)
        {
            try {
                downloadedSoFar += downloadLocations[currentIndex].actualSize;

                ++currentIndex;

                if (currentIndex < downloadLocations.length)
                {
                    startNextDownload();
                }
                else
                {
                    finalizeDownloads();
                }

            } catch (e) {
                unimportantError(e);
                downloadCompleteFolder.remove(true);
                aErrorCallback(e);
            }
        }

        function startNextDownload()
        {
            var location = downloadLocations[currentIndex];
            var url = location.url;{}
{}
{}
downloader.resumeDownloadEnabled = true;
            downloader.temporaryDirectory = downloadTempFolder;
            downloader.deferFinalize = true;

            var progressTracker =
                downloader.download(
                    url,
                    downloadCompleteFolder,
                    progressCallback,
                    successCallback,
                    errorCallback);
            location.progressTracker = progressTracker;
            combinedCall.addCall(progressTracker);
        }

        var finalizeDownloads = makeCallback(this, function()
        {

            for each (let location in downloadLocations)
            {
                if (location.url == null)
                {
                    continue;
                }

                var progressTracker = location.progressTracker;
                var content = location.content;
                var targetFile = progressTracker.finalizeDownload();
                location.url = targetFile;
                if ((content == "icon") ||
                    (content == "preview"))
                {
                    this[content].url = getIOService().newFileURI(targetFile).spec;
                }
            }

            var tocfile = downloadCompleteFolder.clone();
            var fileName = makeSafe.filename(this._mainLocation.fileName);
            fileName = fileName.replace(/\..*/, "") + ".toc";
            tocfile.append(fileName);
            var toc = this.createTOC(getIOService().newFileURI(tocfile));
            var tocXML = toc.toXMLString();{}
writeStringToFile(tocfile, tocXML);
            var addItemsListener =
            {
                started : function()
                {
                },
                progress : function(percent)
                {
},
                addedItem : function(dllitem)
                {
                    dllitem = dllitem.QueryInterface(CI.ttIItem);{}
result.importFromDLL(dllitem);
                    aSuccessCallback(result);
                },
                done : function(errorCode)
                {
},
                error : function(error, resume)
                {
aErrorCallback(error);
                },
                QueryInterface : function(iid)
                {
                    if (iid.equals(CI.ttIAddListener) ||
                        iid.equals(CI.ttIProgressListener) ||
                        iid.equals(CI.nsISupports))
                        return this;
                    throw Components.results.NS_NOINTERFACE;
                }
            };

            var localItems = gApplication.homeServices.localNavItems();
            localItems.addItemsFromTOCFile(
                new ArrayIterator([tocfile]),
                addItemsListener);{}
});

        startNextDownload();

        return combinedCall;
    },
cloneItem : function()
    {
        assert(this._dll, "need local item");

        var clonedItem = new this._listEntry.modelType();
        clonedItem.importFromDLL(this._dll);

        return clonedItem;
    },
getCertificateLocation : function()
    {
        for each(let location in this._locations)
            if (location.content == "activation" || location.content == "durham")
                return location;

        return null;
    },
createCopyWithDurhamCertificate : function(certificateFile)
    {
        assert(this._drm && this._drmScheme == "durham", "item using Durham encryption scheme expected")
        var item = this.cloneItem();

        var certificateLocation = {url : certificateFile, size: certificateFile.fileSize, content: "durham"};

        if (item.locationType == 1)
        {
            item.locations = [certificateLocation];
        }
        else
        {
            item._locations = this._locations.filter(function(location) location.content != "durham" && location.content != "activation");
            item._locations.push(certificateLocation);
        }

        item.drmNeedsKey = false;
        item.drmDeviceCode = gApplication.deviceManager.activeDevice.deviceCode;
        return item._createDLLItem(true);
    }

}
extend(ttNavItem, ttItem);
ttNavItem._getInstaller = function(target, installItems, removeItems)
{
for each (let item in installItems)
    {
        assert(item instanceof ttNavItem, "need ttNavItem");
        item._ensureSourceDLL();{}
}
    var dllitems = new ArrayIterator(installItems, function(item) item.sourceDLL);
    return target.createLocalToDeviceInstaller(dllitems)
                 .QueryInterface(CI.ttIInstaller);
}
ttNavItem.getItemsByTargetLocation = function(items, devItems)
{
    let devItemMap = {__proto__: null};
    for each (let item in devItems)
    {
        let installLocation = item.sourceDLL.installLocation.path.path;
        for each (let id in item.significantIDs)
        {
            if (!(id in devItemMap))
                devItemMap[id] = [];
            devItemMap[id].push(installLocation);
        }
    }
    let itemsMap = {__proto__: null};
    let defaultLocation = gApplication.deviceManager.activeInstallLocation.path.path;
    for each (let item in items)
    {
        let targetLocations = [];

        if (item instanceof ttProgram)
        {
            for each (let installLocation in gApplication.deviceManager.getInstallLocationsForDevice())
                if (installLocation.isBootLocation)
                    targetLocations.push(installLocation.path.path);
        }

        if (targetLocations.length == 0)
        {
            for each (let id in item.significantIDs)
            {
                if (id in devItemMap)
                {
                    alreadyExists = true;
                    for each (let installLocation in devItemMap[id])
                        targetLocations.push(installLocation);
                }
            }
        }

        if (targetLocations.length == 0)
        {
            targetLocations.push(defaultLocation);
        }

        for each (let installLocation in targetLocations)
        {
            if (!(installLocation in itemsMap))
                itemsMap[installLocation] = [];
            if (itemsMap[installLocation].indexOf(item) < 0)
                itemsMap[installLocation].push(item);
        }
    }
    return itemsMap;
}
ttNavItem.getInstalledSizeDelta = function(installItems, removeItems, devItems)
{
    let installLocations = {__proto__ : null};
    let sizeDelta = {__proto__ : null};
    for each (let installLocation in gApplication.deviceManager.getInstallLocationsForDevice())
    {
        installLocations[installLocation.path.path] = installLocation;
        sizeDelta[installLocation.path.path] = 0;
    }

    let installTargets = ttNavItem.getItemsByTargetLocation(installItems, devItems);

    for (let target in installTargets)
    {
        let installer = ttNavItem._getInstaller(installLocations[target], installTargets[target], removeItems);

        for (let installLocationPath in installLocations)
            sizeDelta[installLocationPath] += installer.sizeDeltaInBytes(installLocations[installLocationPath]);
    }
    if (removeItems)
    {
        let iterator = new ArrayIterator(removeItems, function(item) item.sourceDLL);
        let deleter = gApplication.deviceManager.activeInstallLocation.deviceNavItems().createItemDeleter(iterator);
        for (let installLocationPath in installLocations)
            sizeDelta[installLocationPath] -= deleter.sizeInBytesDelta(installLocations[installLocationPath]);
    }

    return sizeDelta;
}
ttNavItem.willItemsFit = function(installItems, removeItems, devItems, additionalSizeRequirements, freeSpaceBuffer)
{
    let sizeDelta = ttNavItem.getInstalledSizeDelta(installItems, removeItems, devItems);

    for each (let installLocation in gApplication.deviceManager.getInstallLocationsForDevice())
    {
        let installLocationPath = installLocation.path.path;
        if (installLocationPath in sizeDelta)
        {
            let available = installLocation.sizeInBytesFree;
            let required = freeSpaceBuffer + sizeDelta[installLocationPath];
            if (additionalSizeRequirements && installLocationPath in additionalSizeRequirements)
                required += additionalSizeRequirements[installLocationPath];

            if (required > available)
                return false;
        }
    }
    return true;
}
ttNavItem.installOnDeviceCheck = function(installItems, removeItems, devItems)
{
    let installLocations = {__proto__ : null};
    for each (let installLocation in gApplication.deviceManager.getInstallLocationsForDevice())
        installLocations[installLocation.path.path] = installLocation;

    let installTargets = ttNavItem.getItemsByTargetLocation(installItems, devItems);
    for (let target in installTargets)
    {
        let installer = ttNavItem._getInstaller(installLocations[target], installItems, removeItems);
        let errorStrOut = {};
        if (!installer.checkPreconditionsOk({value: null}, errorStrOut))
            throw errorStrOut.value;
    }
}
ttNavItem.installOnDevice = function(target, items, progressCallback, successCallback, errorCallback)
{
for (var i = 0; i < items.length; i++)
{
var item = items[i];
assert(item instanceof ttNavItem, "need ttNavItem");
assert(item.isLocal, "must be a local item");{}
}
for each (let item in items)
    {
        item._saveMetadata();
    }

    var dllitems = new ArrayIterator(items, function(item) { return item.sourceDLL; });
    var installer = target.createLocalToDeviceInstaller(dllitems)
                    .QueryInterface(CI.ttIInstaller);
    var errorStrOut = {value: null};
    if (!installer.checkPreconditionsOk({value: null}, errorStrOut))
    {
        errorCallback(errorStrOut.value);
        return new NoOpCall();
    }

    return new Abortable(installer.doInstall(new CopyListener(progressCallback, successCallback, errorCallback)));
}
ttNavItem.copyToPCCheck = function(items, successCallback, errorCallback)
{
    if (items.length == 0)
    {
        successCallback();
        return;
    }
    var targetFolder = gApplication.settings.fromdevItemsDir;
    var target = gApplication.homeServices.localNavItems()
                 .QueryInterface(CI.ttILocalItems);
    var iterator = new ArrayIterator(items, function(item) { return item.sourceDLL; });
    var installer = target.createDeviceToLocalInstaller(iterator, targetFolder)
                    .QueryInterface(CI.ttIInstaller);
    var errorStrOut = {value: null};
    if (!installer.checkPreconditionsOk({value: null}, errorStrOut))
        errorCallback(errorStrOut.value);
    else
        successCallback();
}

ttNavItem.copyToPC = function(items, progressCallback, successCallback, errorCallback)
{
    var targetFolder = gApplication.settings.fromdevItemsDir;
    var target = gApplication.homeServices.localNavItems()
                 .QueryInterface(CI.ttILocalItems);
    var iterator = new ArrayIterator(items, function(item) { return item.sourceDLL; });
    var installer = target.createDeviceToLocalInstaller(iterator, targetFolder)
                    .QueryInterface(CI.ttIInstaller);
    var errorStrOut = {value: null};

    if (!installer.checkPreconditionsOk({value: null}, errorStrOut))
    {
        errorCallback(errorStrOut.value);
        return new NoOpAbortable();
    }
    return new Abortable(installer.doInstall(new CopyListener(progressCallback, successCallback, errorCallback)));
}
