/*
  @Copyright 2006-2009 TomTom, BV
  TomTom is hiring experienced mozilla developers! - contact us at xul-jobs@tomtom.com
*/


var gMainWindow = top.gMainWindow;
function ttTask()
{
    this._cleanupHandlers = [];
    window.addEventListener("DOMContentLoaded", function() {
        let className = " taskDocument";
        if (/\bwindowDialog\b/.test(top.document.documentElement.className))
            className += " windowDialog";
        document.documentElement.className += className;
    }, false);
}
ttTask.prototype =
{
    name : null,
    currentlyNeedDevice : false,
    currentlyNeedLogin : false,
    _ttwizard : null,
    _doneCallback : null,
    _currentDevice : null,
    _deviceObserver : null,
    _loginObserver : null,
    _contentType : null,
    _dialogs : null,
_cleanupHandlers : null,
    _destroyed : false,

    init : function(doneCallback)
    {
        var wizs = document.getElementsByTagName("ttwizard");
        assert(wizs.length == 1, "There must be exactly one <ttwizard> on the task .xul");
        this._ttwizard = wizs[0];
        this._ttwizard.task = this;
        assert(doneCallback, "Need something to continue after task");
        this._doneCallback = doneCallback;
        this._dialogs = [];

        this._deviceObserver = new ActiveDeviceObserver(this);
        this._loginObserver = new LoginObserver(this);
        gApplication.deviceManager.addObserver(this._deviceObserver);
        gApplication.login.addObserver(this._loginObserver);
    },
    start : function(args)
    {
},

    taskDone : function(data)
    {
        var me = this;
        this._ttwizard.startCommit(function(success)
            {
                me.commitFinished(success, data);
            });
    },
commitFinished : function(success, data)
    {
        if (typeof data == "undefined")
            data = null;

        this._destructor(success, data);{}
this._doneCallback(data);
    },

    _destructor : function(success, result)
    {
        if (this._destroyed)
            return;
        this._destroyed = true;

        try
        {
            this._ttwizard.cancelCommit();
            this._ttwizard.cancelNetworkCalls();
        }
        catch (e)
        {
            error(e);
        }
        finally
        {
            gApplication.deviceManager.removeObserver(this._deviceObserver);
            gApplication.login.removeObserver(this._loginObserver);
            this._deviceObserver = null;
            this._loginObserver = null;
            this.abortDialogs();

            window.task = null;

            for each (let handler in this._cleanupHandlers)
            {
                try {
                    handler(this, success, result);
                } catch (e) { error(e); }
            }
        }
    },

    _debugDestructed : function()
    {
        assert(!this._deviceObserver, "task \"" + this.name + "\" not properly closed", "TTTask/DestructionFailure");
    },
addCleanupHandler : function(handler)
    {
        assert(typeof handler == "function", "Cleanup handler must be a function");
        this._cleanupHandlers.push(handler);
    },

    loadSubTask : function(subTask, inArgs, doneCallback)
    {
        try {
            subTask.init(this._ttwizard, inArgs, doneCallback, this);
            subTask.start(inArgs);
        } catch (e) { errorAbortTask(e); }
    },
interrupt : function(interruptCallback, reason, newTaskName)
    {
        assert(typeof interruptCallback == "function", "Need a callback to continue after interrupt");{}
var wiz = this._ttwizard;
        if ((wiz._historyStack.length <= 1 && !wiz.isCommiting) || wiz.commitDone)
        {
            this._destructor(false, null);
            interruptCallback();
            return true;
        }

        var prompt = "";

        if (reason && reason.text)
        {
            prompt += reason.text;
        }

        var stringServ = Cc["@mozilla.org/intl/stringbundle;1"].getService(CI.nsIStringBundleService);
        var stringBundle = stringServ.createBundle("chrome://tthome/locale/ui/bindings/ttwizard.properties");

        prompt = stringBundle.formatStringFromName("interrupt.wizard.desc", [prompt], 1);

        var promptServ = promptService();

        var flags = promptServ.BUTTON_POS_0 * promptServ.BUTTON_TITLE_IS_STRING +
            promptServ.BUTTON_POS_1 * promptServ.BUTTON_TITLE_IS_STRING;

        var ret = promptServ.confirmEx(
                window,
                stringBundle.GetStringFromName("interrupt.wizard.title"),
                prompt,
                flags,
                stringBundle.GetStringFromName("interrupt.wizard.abort"),
                stringBundle.GetStringFromName("interrupt.wizard.stay"),
                null,
                null,
                {value: false}
                );

        if (ret == 0)
        {
            this._destructor(false, null);
            interruptCallback();
            return true;
        }
        else
        {
            return false;
        }
    },
abort : function()
    {
this._destructor(false, null);
        this._doneCallback(null);
    },
trackDialog : function(win)
    {
        var weakRef = Components.utils.getWeakReference(win);
        this._dialogs.push(weakRef);
    },

    abortDialogs : function()
    {
        for (var i = this._dialogs.length - 1; i >= 0; i--)
        {
            var win = this._dialogs[i].get();
            if (win)
            {
                try {
                    gMainWindow.dialogs.closeDialog(win);
                } catch (e) { unimportantError(e); }
            }
            else
            {
                this._dialogs.splice(i, 1);
            }
        }
    },
needDevice : function(successCallback)
    {
        if (gApplication.deviceManager.activeDevice)
        {
            this._currentDevice = gApplication.deviceManager.activeDevice;
            this.currentlyNeedDevice = true;
            this._ttwizard.registerReversibleAction(makeCallback(this, this.noLongerNeedDevice));
            successCallback();
            return;
        }

        this.currentlyNeedDevice = false;
        var me = this;
        this.trackDialog(top.gMainWindow.dialogs.openDialogAsPage(
            top.gMainWindow.dialogs.kDeviceRequiredPage,
            null,
            null,
            function()
            {
if (me._destroyed)
                    return;

                if (gApplication.deviceManager.activeDevice)
                {
                    me._currentDevice = gApplication.deviceManager.activeDevice;
                    me.currentlyNeedDevice = true;
                    me._ttwizard.registerReversibleAction(makeCallback(me, me.noLongerNeedDevice));
                    successCallback();
                    return;
                }
                else
                {
                    me.abort();
                    return;
                }
            },
            false,
            top));
    },

    noLongerNeedDevice : function()
    {
        this._currentDevice = null;
        this.currentlyNeedDevice = false;
    },

    _observerDeviceChanged : function(deviceChanged, locationChanged)
    {
        if (!deviceChanged && !locationChanged)
        {
            return;
        }
        if (!this.currentlyNeedDevice)
        {
            return;
        }
        if (gApplication.deviceManager.activeDevice)
        {
            if (gApplication.deviceManager.activeDevice == this._currentDevice)
                this._installLocationChanged();
            else
            {
                this._currentDevice = gApplication.deviceManager.activeDevice;
                this._deviceChanged();
            }
        }
        else
        {
            this._deviceRemoved();
        }
    },
_deviceRemoved : function()
    {
        this._handleDeviceChange();
    },
_deviceChanged : function()
    {
        this._handleDeviceChange();
    },
_installLocationChanged : function()
    {
        this._handleDeviceChange();
    },
_handleDeviceChange : function()
    {
        if (this._ttwizard && this._ttwizard.commitDone)
        {
            task.abort();
            return;
        }

        var stringBundle = getStringBundle("chrome://tthome/locale/logic/ttTask.properties");

        if (this._ttwizard && this._ttwizard.isCommiting)
        {
            this._ttwizard.cancelCommit();
            errorNonFatal(new InformativeException(stringBundle.GetStringFromName("deviceDisconnectedError.prompt")));
            return;
        }

        errorAbortTask(new InformativeException(stringBundle.GetStringFromName("deviceDisconnectedError.prompt")));
    },
doDeviceScan : function(successCallback, errorCallback, itemList)
    {
        assert(typeof successCallback == "function", "successCallback must be a function");
        assert(typeof errorCallback == "function", "errorCallback must be a function");
        assert(!itemList || itemList instanceof ttItemList, "itemList must be a ttItemList");

        this._doDeviceScanInternal(successCallback, errorCallback, itemList, gApplication.deviceManager.getInstallLocationsForDevice(), null);
    },

    _doDeviceScanInternal: function(successCallback, errorCallback, itemList, installLocations, doneLocation)
    {
        if (itemList && doneLocation)
        {
            try {
                itemList.addItemListFromDLL(doneLocation.deviceNavItems());
            } catch (e) { error(e); }
            try {
                itemList.addItemListFromDLL(doneLocation.devicePhotos());
            } catch (e) { error(e); }
            try {
                itemList.addItemListFromDLL(doneLocation.deviceMusic());
            } catch (e) { error(e); }
        }

        if (installLocations.length > 0)
        {
            let installLocation = installLocations.shift();
            installLocation.deviceNavItems().populate(
                new ScanListener(function() {},
                        makeCallback(this, this._doDeviceScanInternal, successCallback, errorCallback, itemList, installLocations, installLocation),
                        errorCallback)
            );
        }
        else
        {
            successCallback();
        }
    },

needLogin : function(successCallback, cancelCallback)
     {
        if (gApplication.login.isLoggedIn)
        {
            this.currentlyNeedLogin = true;
            this._ttwizard.registerReversibleAction(makeCallback(this, this.noLongerNeedLogin));
            successCallback();
            return new NoOpCall();
        }

        this.currentlyNeedLogin = false;
        var me = this;
        return gApplication.login.tryStoredLogin(function()
        {
            me.currentlyNeedLogin = true;
            me._ttwizard.registerReversibleAction(makeCallback(me, me.noLongerNeedLogin));
            successCallback();
            return;
        },
        function()
        {
            top.gMainWindow.dialogs.openDialog(
                "chrome://tthome/content/ui/dialogs/taskDialog/taskDialog.xul", {
                    url: "chrome://tthome/content/ui/task/login/login.xul",
                    taskParams: { loginOptional: false }
                },
                null,
                function()
                {
                    if (me._destroyed)
                        return;

                    if (gApplication.login.isLoggedIn)
                    {
                        me.currentlyNeedLogin = true;
                        me._ttwizard.registerReversibleAction(makeCallback(me, me.noLongerNeedLogin));
                        successCallback();
                        return;
                    }
                    else
                    {
                        if (!cancelCallback)
                            me.abort();
                        else
                            cancelCallback();
                        return;
                    }
                },
                false,
                window);
        },
        errorAbortTask);
     },
shouldLogin : function(successCallback, deniedCallback)
     {
        if (gApplication.login.isLoggedIn)
        {
            successCallback();
            return new NoOpCall();
        }
        if (gApplication.login.shouldLoginDenied)
        {
            deniedCallback();
            return new NoOpCall();
        }

        var old = this.currentlyNeedLogin;
        this.currentlyNeedLogin = false;
        var me = this;
        return gApplication.login.tryStoredLogin(successCallback, function()
        {
            top.gMainWindow.dialogs.openDialog(
                "chrome://tthome/content/ui/dialogs/taskDialog/taskDialog.xul", {
                    url: "chrome://tthome/content/ui/task/login/login.xul",
                    taskParams: { loginOptional: true }
                },
                null,
                function()
                {
                    if (me._destroyed)
                        return;

                    me.currentlyNeedLogin = old;
                    if (gApplication.login.isLoggedIn)
                    {
                        successCallback();
                        return;
                    }
                    else
                    {
                        gApplication.login.shouldLoginDenied = true;
                        if (deniedCallback)
                        {
                            deniedCallback();
                        }
                        return;
                    }
                },
                false,
                window);
        },
        deniedCallback);
     },

     noLongerNeedLogin : function()
     {
        this.currentlyNeedLogin = false;
     },

     _observerLoginChanged : function()
     {
        if (!this.currentlyNeedLogin)
        {
            return;
        }
        if (gApplication.login.isLoggedIn)
        {
            this._usernameChanged();
        }
        else
        {
            this._loggedOut();
        }
     },
     _loggedOut : function()
     {
        this.needLogin(function()
        {
        });
     },
     _usernameChanged : function()
     {
        if (!this.currentlyNeedLogin)
            return;
        var stringBundle = getStringBundle("chrome://tthome/locale/logic/ttTask.properties");
        ttMessageService().alert(
            stringBundle.GetStringFromName("loginChangedWarning.title"),
            stringBundle.GetStringFromName("loginChangedWarning.prompt"));
     },

    __end : null
}
function ActiveDeviceObserver(task)
{
    this._task = task;
}
ActiveDeviceObserver.prototype =
{
    changed : function(deviceChanged, locationChanged)
    {
        this._task._observerDeviceChanged(deviceChanged, locationChanged);
    }
}
function LoginObserver(task)
{
    this._task = task;
}
LoginObserver.prototype =
{
    changed : function()
    {
        this._task._observerLoginChanged();
    }
}

function errorAbortTask(e)
{
    task.abort();
    error(e);
}

function errorNonFatal(e)
{
    if (e)
    {
        e = ensureExceptionIsObject(e);
        e.severity = Exception.downgradeSeverity(e.severity, Exception.SEVERITY_WARNING);
    }

    error(e);
}
