/*
  @Copyright 2006-2009 TomTom, BV
  TomTom is hiring experienced mozilla developers! - contact us at xul-jobs@tomtom.com
*/



function LoginSubTask(inputData)
{
    ttSubTask.apply(this, arguments);
    this._stringBundle = getStringBundle("chrome://tthome/locale/ui/subtask/login/loginSubTask.properties");
}
LoginSubTask.prototype =
{
    name: "login",
    _stringBundle: null,
    _inputData: null,
start: function(inputData)
    {
        ttSubTask.prototype.start.apply(this, arguments);

        this._inputData = inputData;

        if (inputData["createAccount"])
            this._newAccount();
        else
            this._loginSubTaskLoad();
    },
_loginSubTaskLoad: function()
    {
        this._ttwizard.loadPage(
            this._constructPageURL("loginPage"),
            this._inputData,
            makeCallback(this, this._loginPageAsyncLoad),
            makeCallback(this, this._loginPageCheck),
            makeCallback(this, this._loginPageDone));
    },
_loginPageAsyncLoad: function(pageHandle)
    {
        pageHandle.inputDone();
    },
_loginPageCheck: function(result, successCallback, errorCallback, returnCallback)
    {
        var disposition = result["disposition"]
        assert(disposition == "login" || disposition == "newAccount", "Unknown or undefined disposition.");
        if (disposition == "login")
        {
            this._tryLogin(result, successCallback, errorCallback, returnCallback);
        }
        else if (disposition == "newAccount")
        {
            successCallback();
        }
    },
_loginPageDone: function(result)
    {
        var disposition = result["disposition"];
        assert(disposition == "login" || disposition == "newAccount", "Unknown or undefined disposition.");
        if (disposition == "newAccount")
        {
            this._newAccount();
        }
        else
        {
            this.subtaskDone();
        }
    },
_tryLogin: function(result, successCallback, errorCallback, returnCallback)
    {
        var username = result["username"];
        var password = result["password"];
        var remember = result["rememberPassword"];

        this.networkCall(gApplication.login.login(username, password,
        makeCallback(this, function()
            {
                try
                {
                    this._rememberPassword(remember, username, password);
                    successCallback();
                } catch (e) {
                    errorCallback(e);
                }
            }),
        makeCallback(this, function(e)
            {
                try {
                    if ((e.code == 401) || (e.code == 104))
                    {
                        var text = this._stringBundle.formatStringFromName("error401Login", [username], 1);
                        var title = this._stringBundle.GetStringFromName("error401LoginTitle");
                        ttMessageService().alert(title, text);
                    }
                    else if (e.code == 402)
                    {
                        var text = this._stringBundle.formatStringFromName("error402Login", [username], 1);
                        var title = this._stringBundle.GetStringFromName("error402LoginTitle");
                        ttMessageService().alert(title, text);
                    }
                    else
                    {
                        errorCallback(e);
                        return;
                    }
                } catch (e) {
                    errorCallback(e);
                    return;
                }
                returnCallback();
            })));
    },
_rememberPassword: function(remember, username, password)
    {
        var dev = gApplication.deviceManager.activeDevice;
        try {
            if (dev)
                prefService().setCharPref("tomtom.device." + dev.id + ".username", username);
            prefService().setCharPref("tomtom.account.username", username);
        } catch (e) { unimportantError(e); }

        try {
            prefService().setBoolPref("tomtom.account." + username + ".rememberPassword", remember);
            prefService().setBoolPref("tomtom.account.rememberPassword", remember);

            if (remember)
            {
                gApplication.login.savePassword(username, password);
            }
            else
            {
                gApplication.login.deletePassword(username);
            }
        } catch (e) {
            error (e);
        }
    },
_newAccount: function()
    {
        this._ttwizard.loadPage(
            this._constructPageURL("newAccountPage"),
            this._inputData,
            makeCallback(this, this._newAccountPageAsyncLoad),
            makeCallback(this, this._newAccountPageCheck),
            makeCallback(this, this._newAccountPageDone));
    },
_newAccountPageAsyncLoad: function(pageHandle)
    {
        pageHandle.inputDone();
    },
_newAccountPageCheck: function(result, successCallback, errorCallback, returnCallback)
    {
        var username = result['username'];
        var password = result['password'];
        var remember = result['rememberPassword'];
        var tomtomNews = result['tomtomNews'];
        var tomtomWork = result['tomtomWork'];
        var publicBeta = prefService().getBoolPref("tomtom.app.showBetaVersions");

        this.networkCall(rpc.call("account", "checkUsernameExists", {
            username: username
        }, null, RPC.METHOD_GET & ~RPC.USE_CACHE,
        makeCallback(this, function(result)
        {
            if (result.exists[0] == "true")
            {
                ttMessageService().alert(this._stringBundle.GetStringFromName("username"),
                                         this._stringBundle.GetStringFromName("usernameExists"));
                returnCallback();
                return;
            }

            this.networkCall(rpc.call("account", "createAccount", {
                username: username,
                password: password,
                emailAddress: username,
                publicBeta: publicBeta
            }, null, RPC.METHOD_POST,
            makeCallback(this, function()
            {
                this.networkCall(rpc.login(username, password,
                makeCallback(this, function()
                {
                    this._rememberPassword(remember, username, password);

                    this.networkCall(rpc.call("account", "updateAccount", {
                        newsletter: tomtomNews,
                        worknewsletter: tomtomWork,
                        publicBeta: publicBeta
                    }, null, RPC.METHOD_POST,
                    makeCallback(this, function()
                    {
                        this.networkCall(gApplication.login.login(username, password,
                        makeCallback(this, function()
                        {
                            successCallback();
                        }),
                        makeCallback(this, function(e)
                        {
                            errorCallback(e);
                        })));
                    }),
                    makeCallback(this, function(e)
                    {
                        errorCallback(e);
                    })));
                }),
                makeCallback(this, function(e)
                {
errorCallback(e);
                })));
            }),
            makeCallback(this, function (e)
            {
                errorCallback(e);
            })));

        }),
        makeCallback(this, function(e)
        {
            errorCallback(e);
        })));
    },
_newAccountPageDone: function(result)
    {
        this.subtaskDone();
    }

}
extend(LoginSubTask, ttSubTask);
